/*********************************************************************
 * LCD Manager for Desktop Station main
 *
 * Copyright (C) 2015 Yaasan
 *
 */

#include <Arduino.h>
#include <avr/pgmspace.h>
#include "OzOLED.h"
#include "DSCoreM_Type.h"
#include "DSCoreM_Common.h"
#include "DSLCDManager.h"
#include "DSSequence.h"
#include <string.h>

char gText[16 + 1];

const char rMenu0[] PROGMEM = "Loco   ";
const char rMenu1[] PROGMEM = "Turnout";
const char rMenu2[] PROGMEM = "Read CV";
const char rMenu3[] PROGMEM = "WriteCV";
const char rMenu4[] PROGMEM = "Analog ";
const char rMenu5a[] PROGMEM = "MM2mode";
const char rMenu5b[] PROGMEM = "DCCmode";
const char rBlank[] PROGMEM    = "        ";
const char rReading[] PROGMEM  = "Reading.";
const char rWriting[] PROGMEM  = "Writing.";
const char rCantRead[] PROGMEM = "Read Err";
const char rFinWrite[] PROGMEM = "WriteEnd";
const char rFinWriteFail[] PROGMEM = "WriteErr";
const char rNoError[] PROGMEM    = "NO ERRORS";
const char rErrorOC[] PROGMEM    = "ERR: OC ";
const char rErrorOV[] PROGMEM    = "ERR: OV ";
const char rErrorLV[] PROGMEM    = "ERR: LV ";


DSLCDManager::DSLCDManager()
{
	valCurrent_H = 0;
	valCurrent_L = 0;
	valVoltage_H = 0;
	valVoltage_L = 0;
	valErrCurrent_H = 0;
	valErrCurrent_L = 0;
	valErrVoltage_H = 0;
	valErrVoltage_L = 0;
	valErrorFlag = 0;
	
	valPower = 0;
	valAccAddress = 0;
	valAccDirection = 0;
	
	valLocAddress = 0;
	valLocSpeed = 0;
	valLocDirection = 0;
	valLocFunction = 0;
	valLocFuncPower = 0;
	valLocFuncBuf = 0;
	valProtocol = 0;
	
	mode_cv_seq = 0;
	
	
	//Menu
	valSelectedMenuItemNo = 0;
	valConfigItemNo = 0;
	
	mode_display = MODE_DISP_HELLO;
	mode_display_temp = MODE_DISP_TEMP_NONE;
	
	RefreshAll();
	flagRefresh[5] = false;
	flagRefresh[6] = false;
	flagRefresh[7] = false;
	
	for( byte i = 0; i < 8; i++)
	{
		flagRefreshCnt[i] = 0;
	}
	
}

void DSLCDManager::DisplayTop()
{
	if( flagRefresh[REFRESH_FLAG_TOP] == true)
	{
		switch(mode_display)
		{
		case MODE_DISP_ANA:
			sprintf(gText, "ANALOG");
			PrintString(gText, 0, 0);
			break;
		case MODE_DISP_LOC:
			sprintf(gText, "LOCO ");
			PrintString(gText, 0, 0);
			break;
			
		case MODE_DISP_ACC:
			sprintf(gText, "TURNOUT");
			PrintString(gText, 0, 0);
			break;
		case MODE_DISP_CVR:
			sprintf(gText, "CV READ");
			PrintString(gText, 0, 0);
			break;
		case MODE_DISP_CVW:
			sprintf(gText, "CVWRITE");
			PrintString(gText, 0, 0);
			break;
		case MODE_DISP_MENU:
			//Home Icon
			DrawIcon(0, 0, ICON_HOME, false);
			PrintString("HOME---", 1, 0);
			break;
		}
			
		if( valPower == 0)
		{
			DrawIcon(15, 0, 7, false);
		}
		else
		{
			DrawIcon(15, 0, 1, false);
		}
		
		if( mode_display != MODE_DISP_ANA)
		{
			//vgR\(AiO[hȊȌꍇ)
			if( valProtocol == 0)
			{
				PrintString("DCC", 11, 0);
			}
			else
			{
				PrintString("MM2", 11, 0);
			}
		}
		else
		{
			PrintString("   ", 11, 0);
		}
		
		
		flagRefresh[REFRESH_FLAG_TOP] = false;
	}
	
}

void DSLCDManager::SetSensors(unsigned short inI_H, unsigned short inI_L, unsigned char inV_H, unsigned char inV_L, unsigned char inErrFlag)
{
	
	valCurrent_H = inI_H;
	valCurrent_L = inI_L;
	valVoltage_H = inV_H;
	valVoltage_L = inV_L;
	valErrorFlag = inErrFlag;
	
	flagRefresh[REFRESH_FLAG_BTM] = true;
}

void DSLCDManager::SetSensorsErr(unsigned short inI, unsigned char inV)
{
	
	unsigned short aCurrent_H = inI / 100;
	unsigned short aCurrent_L = inI - (aCurrent_H * 100);
	unsigned char aVoltage_H = inV / 10;
	unsigned char aVoltage_L = inV - (aVoltage_H * 10);
	
	valErrCurrent_H = aCurrent_H;
	valErrCurrent_L = aCurrent_L;
	valErrVoltage_H = aVoltage_H;
	valErrVoltage_L = aVoltage_L;
	
}

void DSLCDManager::Interval()
{
	
	if( flagRefresh[REFRESH_FLAG_BTM] == true)
	{
		switch(flagRefreshCnt[REFRESH_FLAG_BTM])
		{
		case 0:
			/* Sensor */
			if( valPower == 0)
			{
					sprintf(gText, "---mA");
			}
			else
			{
				
				if( valCurrent_H > 0)
				{
					sprintf(gText, "%d.%dA", valCurrent_H, valCurrent_L / 10);
				}
				else
				{
					sprintf(gText, "%3dmA", valCurrent_L * 10);
				}
			}
			PrintString(gText, 0, 7);
			
			flagRefreshCnt[REFRESH_FLAG_BTM] = 1;
			break;
		case 1:
			/* Volatge Sensor */
			sprintf(gText, "%d.%dV",  valVoltage_H, valVoltage_L);
			
			PrintString(gText, 6, 7);
			flagRefreshCnt[REFRESH_FLAG_BTM] = 2;
			break;
		case 2:
			/* Error flag */
			if( valErrorFlag > 0)
			{
				if( (valErrorFlag & 0b100) > 0)
				{
					sprintf(gText, "OC");
				}
				else if( (valErrorFlag & 0b010) > 0)
				{
					sprintf(gText, "LV");
				}
				else if( (valErrorFlag & 0b001) > 0)
				{
					sprintf(gText, "OV");
				}
				
				DrawIcon(13, 7, 6, false);
				PrintStringNOT(gText, 14, 7);
			}
			else
			{
				sprintf(gText, "   ");
				PrintString(gText, 13, 7);
			}
			
			flagRefresh[REFRESH_FLAG_BTM] = false;
			flagRefreshCnt[REFRESH_FLAG_BTM] = 0;
			break;
		}
	}
	
	/* TOP CATEGORY & Power Icon */
	DisplayTop();
	
	switch( mode_display)
	{
	case MODE_DISP_ANA:
		//Draw speed bar
		DrawSpeedBar( 5, 3, valLocSpeed >> 2, 0);
		
		if( valLocSpeed < 4)
		{
			PrintStringNOT(" STOP! ", 5, 4);
		}
		else
		{
			sprintf(gText, " %3d%c  ", ((word)(valLocSpeed >> 3) * 100) >> 7, 37);
			PrintStringNOT(gText, 5, 4);
		}
		
		//Direction
		if( valLocDirection == 0)
		{
			PrintString("  FWD", 5, 2);
			DrawIcon(10, 2, 2, false);
			DrawIcon(11, 2, 2, false);
		}
		else
		{
			DrawIcon(5, 2, 3, false);
			DrawIcon(6, 2, 3, false);
			PrintString("REV  ", 7, 2);
		}
		
	break;

	case MODE_DISP_LOC:
		
		//Address
		if( flagRefresh[REFRESH_FLAG_ADDR] == true)
		{
			DisplayLocAddress();
			flagRefresh[REFRESH_FLAG_ADDR] = false;
		}
		
		//SpeedBar
		if( flagRefresh[REFRESH_FLAG_BAR] == true)
		{
			
			switch(flagRefreshCnt[REFRESH_FLAG_BAR])
			{
			case 0:
				//Draw speed bar
				DrawSpeedBar( 9, 3, valLocSpeed >> 2, 0);
				
				flagRefreshCnt[REFRESH_FLAG_BAR] = 1;
				break;
				
			case 1:
				if( valLocSpeed < 4)
				{
					PrintStringNOT(" STOP! ", 9, 4);
				}
				else
				{
					sprintf(gText, " %3d%c  ", ((word)(valLocSpeed >> 3) * 100) >> 7, 37);
					PrintStringNOT(gText, 9, 4);
				}
				
				flagRefresh[REFRESH_FLAG_BAR] = false;
				flagRefreshCnt[REFRESH_FLAG_BAR] = 0;
				break;
			}
		}
			
		if( flagRefresh[REFRESH_FLAG_DIR] == true)
		{
			//Direction
			if( valLocDirection == 0x00)
			{
				PrintString("  FWD", 9, 2);
				DrawIcon(14, 2, 2, false);
				DrawIcon(15, 2, 2, false);
			}
			else
			{
				DrawIcon(9, 2, 3, false);
				DrawIcon(10, 2, 3, false);
				PrintString("REV  ", 11, 2);
			}
			
			flagRefresh[REFRESH_FLAG_DIR] = false;
		}
		
		if( flagRefresh[REFRESH_FLAG_FNC] == true)
		{
			switch( mode_display_temp)
			{
			case MODE_DISP_TEMP_NONE:
				
				for( byte i = 0; i < 16; i++)
				{
					if( ((valLocFuncBuf >> i) & 1) > 0)
					{
						sprintf(gText, "%X", i);
						
						PrintStringNOT( gText, i, 5);
					}
					else
					{
						PrintString(" ", i, 5);
					}
				}
				
				break;
			case MODE_DISP_TEMP_FUNC:
				sprintf(gText, "F%02d %s ", valLocFunction, (valLocFuncPower == 1) ? "ON " : "OFF");
				PrintString(gText, 0, 5);
				break;
			}
			
			flagRefresh[REFRESH_FLAG_FNC] = false;
		}
		
		break;
	
	
	case MODE_DISP_ACC:
		//Address
		if( flagRefresh[REFRESH_FLAG_ADDR] == true)
		{
			DisplayAccAddress();
			flagRefresh[REFRESH_FLAG_ADDR] = false;
		}
		
		//SpeedBar
		if( flagRefresh[REFRESH_FLAG_DIR] == true)
		{
			if( valAccDirection == 0)
			{
				PrintString("Div/", 10, 4);
				DrawIcon(9, 3, 5, false);
				DrawIcon(10, 3, 5, false);
				DrawIcon(11, 3, 5, false);
				DrawIcon(12, 2, 5, false);
				DrawIcon(13, 2, 5, false);
				DrawIcon(14, 2, 5, false);
				DrawIcon(12, 3, 4, false);
				DrawIcon(13, 3, 4, false);
				DrawIcon(14, 3, 4, false);

			}
			else
			{
				PrintString("Str/", 10, 4);
				DrawIcon(12, 2, 4, false);
				DrawIcon(13, 2, 4, false);
				DrawIcon(14, 2, 4, false);
				DrawIcon(9, 3, 5, false);
				DrawIcon(10, 3, 5, false);
				DrawIcon(11, 3, 5, false);
				DrawIcon(12, 3, 5, false);
				DrawIcon(13, 3, 5, false);
				DrawIcon(14, 3, 5, false);
			}
			flagRefresh[REFRESH_FLAG_DIR] = false;
		}
		
		break;
		
		
	case MODE_DISP_CVR:
		
		if( mode_cv_seq == 0)
		{
			sprintf(gText, "CVNo%4d", valCVAddr);
			PrintString(gText, 0, 3);
			sprintf(gText, "Enter to Read.");
			PrintString(gText, 0, 4);
		}
		else if( mode_cv_seq == 10)
		{
			sprintf(gText, "CVNo%4d", valCVAddr);
			PrintString(gText, 0, 3);
			StrCpyFromROM(gText, rReading, 8);
			PrintString(gText, 0, 4);
			
		}
		else if( mode_cv_seq == 1)
		{
			sprintf(gText, "CVNo%4d", valCVAddr);
			PrintString(gText, 0, 3);
			sprintf(gText, "Val=>%3d", valCVValue);
			PrintString(gText, 0, 4);
		}
		else if( mode_cv_seq == 11)
		{
			sprintf(gText, "CVNo%4d", valCVAddr);
			PrintString(gText, 0, 4);
			StrCpyFromROM(gText, rCantRead, 8);
			PrintString(gText, 0, 4);
		}
		
		
		break;
		
	case MODE_DISP_CVW:
		
		/* Row 1 */
		
		if( mode_cv_seq == 0)
		{
			sprintf(gText, "%cCV=%4d", 0x07, valCVAddr);
		}
		else
		{
			sprintf(gText, " CV=%4d", valCVAddr);
		}
		
		PrintString(gText, 0, 3);
		
		
		if( (mode_cv_seq == 0) || ( mode_cv_seq == 1))
		{
			/* Row 2 */
			if( mode_cv_seq == 1)
			{
				sprintf(gText, "%cVal=%3d", 0x07, valCVValue);
			}
			else
			{
				sprintf(gText, " Val=%3d", valCVValue);
			}
			
			PrintString(gText, 0, 4);
		}
		else if( mode_cv_seq == 2)
		{
			
			if( valCVAddr == 0xFFFF)
			{
				//Failed to CV Write
				StrCpyFromROM(gText, rFinWriteFail, 8);
				PrintString(gText, 0, 4);
			}
			else
			{
				//Success CV write
				StrCpyFromROM(gText, rFinWrite, 8);
				PrintString(gText, 0, 4);
			}
		}
		else if( mode_cv_seq == 20)
		{
			StrCpyFromROM(gText, rWriting, 8);
			PrintString(gText, 0, 4);
		}
		
		break;
		
		
	case MODE_DISP_MENU:
		
		for( int j = 0; j < 6; j++)
		{
			
			switch(j)
			{
			case 0: 
				StrCpyFromROM(gText, rMenu0, 8);
				break;
			case 1: 
				StrCpyFromROM(gText, rMenu1, 8);
				break;
			case 2: 
				StrCpyFromROM(gText, rMenu2, 8);
				break;
			case 3: 
				StrCpyFromROM(gText, rMenu3, 8);
				break;
			case 4: 
				StrCpyFromROM(gText, rMenu4, 8);
				break;
			case 5:
				if( valProtocol == 0)
				{
					StrCpyFromROM(gText, rMenu5a, 8);
				}
				else
				{
					StrCpyFromROM(gText, rMenu5b, 8);
				}
				break;
			default:
				StrCpyFromROM(gText, rBlank, 8);
				break;
				
			}
			
			ClearStringSpaces(gText, 15);
			
			if( j == valSelectedMenuItemNo)
			{
				DrawIcon(0, 1 + j, 2, false);
			}
			else
			{
				PrintString(" ", 0, j + 1);
			}
			
			
			PrintString(gText, 1, j + 1);
		}
			
		
		break;
		
	}
	
}


void DSLCDManager::StrCpyFromROM(char *outpText, const char *inpROMText, byte inLen)
{
	byte i;
	
	for( i = 0; i < inLen; i++)
	{
		outpText[i] = (char)pgm_read_byte(inpROMText + i);
	}
	
	for( i = inLen; i < 16; i++)
	{
		outpText[i] = 0x00;
	}
}

void DSLCDManager::DisplayLocAddress()
{
	word aProtocol = GetLocIDProtocol(valLocAddress >> 8);
	
	sprintf(gText, "%04d", valLocAddress - aProtocol);
	PrintStringBig(gText, 0, 1);
	
}

void DSLCDManager::DisplayAccAddress()
{
	word aProtocol = GetLocIDProtocol(valAccAddress >> 8);
	
	word aAddr = ((valProtocol == 0) ? 1 : 0) + valAccAddress - aProtocol;
	
	sprintf(gText, "%04d", aAddr);
	PrintStringBig(gText, 0, 1);
	
}

void DSLCDManager::RegisterLocSpeed(word inLocAddr, word inSpeed)
{
	valLocAddress = inLocAddr;
	valLocSpeed = inSpeed;
	
	flagRefresh[REFRESH_FLAG_BAR] = true;
	
}

void DSLCDManager::RegisterLoc(word inLocAddr, unsigned char inDir, unsigned long inFuncBuf)
{
	valLocAddress = inLocAddr;
	valLocDirection = inDir;
	valLocFuncBuf = inFuncBuf;
	
	
	mode_display = MODE_DISP_LOC;
	flagRefresh[REFRESH_FLAG_DIR] = true;
	flagRefresh[REFRESH_FLAG_ADDR] = true;
	flagRefresh[REFRESH_FLAG_FNC] = true;
	
}

void DSLCDManager::RegisterLocDirection(word inLocAddr, unsigned char inDir)
{
	valLocAddress = inLocAddr;
	valLocDirection = inDir;
	
	flagRefresh[REFRESH_FLAG_DIR] = true;
	
}

void DSLCDManager::RegisterLocFunctionNormal(word inLocAddr, unsigned long inFuncBuf)
{
	valLocAddress = inLocAddr;
	valLocFuncBuf = inFuncBuf;
	
	flagRefresh[REFRESH_FLAG_FNC] = true;
	mode_display_temp = MODE_DISP_TEMP_NONE;
}

void DSLCDManager::RegisterLocFunction(word inLocAddr, unsigned char inFuncNo, unsigned char inFuncPower)
{
	valLocAddress = inLocAddr;
	valLocFunction = inFuncNo;
	valLocFuncPower = inFuncPower;
	
	flagRefresh[REFRESH_FLAG_FNC] = true;
	mode_display_temp = MODE_DISP_TEMP_FUNC;
}

void DSLCDManager::RegisterPower(unsigned char inPower)
{
	valPower = inPower;
	
	flagRefresh[REFRESH_FLAG_TOP] = true;
	
}

void DSLCDManager::RegisterTurnoutDir(word inAccAddr, unsigned char inDir)
{
	word aLocProtocol = GetLocIDProtocol(inAccAddr >> 8);
	
	if( valAccAddress == inAccAddr)
	{
		valAccDirection = inDir;
		flagRefresh[REFRESH_FLAG_DIR] = true;
	}
}

void DSLCDManager::RegisterTurnout(word inAccAddr, unsigned char inDir)
{
	valAccAddress = inAccAddr;
	valAccDirection = inDir;
	
	if( mode_display != MODE_DISP_ACC)
	{
		mode_display = MODE_DISP_ACC;
		flagRefresh[REFRESH_FLAG_TOP] = true;
	}
	
	
	flagRefresh[REFRESH_FLAG_ADDR] = true;
	flagRefresh[REFRESH_FLAG_DIR] = true;
	
}

void DSLCDManager::RegisterMenu(byte inMenuItemNo)
{
	
	valSelectedMenuItemNo = inMenuItemNo;
	mode_display = MODE_DISP_MENU;
	
	//ALL
	RefreshAll();
	
}

void DSLCDManager::RegisterMain()
{
	
	
	mode_display = MODE_DISP_LOC;
	
	//ALL
	RefreshAll();
	
}

void DSLCDManager::RegisterAnalog()
{
	
	
	mode_display = MODE_DISP_ANA;
	
	//ALL
	RefreshAll();
	
}

void DSLCDManager::RegisterProtocol(byte inProtocol)
{
	
	valProtocol = inProtocol;
	
	//ALL
	RefreshAll();
	
}

unsigned char DSLCDManager::GetMode()
{
	return mode_display;
}

void DSLCDManager::RefreshAll()
{
	flagRefresh[REFRESH_FLAG_ADDR] = true;
	flagRefresh[REFRESH_FLAG_TOP] = true;
	flagRefresh[REFRESH_FLAG_BAR] = true;
	flagRefresh[REFRESH_FLAG_FNC] = true;
	flagRefresh[REFRESH_FLAG_BTM] = true;
	flagRefresh[REFRESH_FLAG_DIR] = true;
	
	
}


void DSLCDManager::RegisterCVWrite(byte inSeq, word inAddr, byte inValue)
{
	valCVAddr = inAddr;
	valCVValue= inValue;
	
	mode_display = MODE_DISP_CVW;
	mode_cv_seq = inSeq;
	
	//ALL
	RefreshAll();
	
}

void DSLCDManager::RegisterCVRead(byte inSeq, word inAddr, byte inValue)
{
	valCVAddr = inAddr;
	valCVValue= inValue;
	
	mode_display = MODE_DISP_CVR;
	mode_cv_seq = inSeq;
	
	//ALL
	RefreshAll();
	
}

word DSLCDManager::GetLocIDProtocol(byte address)
{
	if( address < 0x04)
	{
			return ADDR_MM2;
	}
	else if( (address >= 0x30) && (address <= 0x33))
	{
			return ADDR_ACC_MM2;
	}
	else if( (address >= 0x38) && (address <= 0x3F))
	{
			return ADDR_ACC_DCC;
	}
	else if( (address >= 0x40) && (address <= 0x70))
	{
			return ADDR_MFX;
	}
	else if( (address >= 0xC0) && (address <= 0xFF))
	{
			return ADDR_DCC;
	}
	else
	{
		return 0;
	}
}

void DSLCDManager::ClearStringSpaces(char *inText, byte inLen)
{
	byte aChk = 0;
	
	for(int i = 0; i < inLen; i++)
	{
		if( (inText[i] == 0x00) || (aChk == 1))
		{
			aChk = 1;
			inText[i] = ' ';
		}
		else
		{
			//Nothing to do
			//inText[i] = inText[i];
		}
	}
}


void DSLCDManager::WriteLocAddr(word inAddr)
{
	valLocAddress = inAddr;
	
}

void DSLCDManager::WriteAccAddr(word inAddr)
{
	valAccAddress = inAddr;
	
}

void DSLCDManager::clearTextBuf()
{
	for( int i = 0; i < 16; i++)
	{
		gText[i] = 0;
	}
}


