/*********************************************************************
 * LCD management process for Desktop Station main
 *
 * Copyright (C) 2015 Yaasan
 *
 */
 
#ifndef DSLCDMANAGER_H
#define DSLCDMANAGER_H

#include <Arduino.h>

#define MODE_DISP_HELLO			0
#define MODE_DISP_LOC			1
#define MODE_DISP_ACC			6
#define MODE_DISP_MENU			7
#define MODE_DISP_CVR			2
#define MODE_DISP_CVW			3
#define MODE_DISP_INF			4
#define MODE_DISP_CFG			5
#define MODE_DISP_ANA			8
#define MODE_DISP_ERR			99

#define MODE_DISP_TEMP_NONE		0
#define MODE_DISP_TEMP_FUNC		10

#define MODE_TOP_LOC		0
#define MODE_TOP_ACC		1

#define REFRESH_FLAG_ADDR	1
#define REFRESH_FLAG_TOP	0
#define REFRESH_FLAG_BAR	2
#define REFRESH_FLAG_FNC	3
#define REFRESH_FLAG_BTM	4
#define REFRESH_FLAG_DIR	5


typedef void (*PrintFunction) (const char *s, const int x, const int y);
typedef void (*DrawFunction) (byte x, byte y, byte num, byte inNot);
typedef void (*VoidFunction) ();
typedef byte (*GetFunction) (byte inNo);

class DSLCDManager
{

	private:
		
	
	//一時保存関連
	unsigned short valCurrent_H;
	unsigned short valCurrent_L;
	unsigned char valVoltage_H;
	unsigned char valVoltage_L;
	unsigned short valErrCurrent_H;
	unsigned short valErrCurrent_L;
	unsigned char valErrVoltage_H;
	unsigned char valErrVoltage_L;

	unsigned char valErrorFlag;
	bool flagRefresh[8];
	byte flagRefreshCnt[8];
	unsigned char valPower;
	word valAccAddress;
	unsigned char valAccDirection;
	word valLocAddress;
	word valLocSpeed;
	word valProtocol;
	unsigned char valLocDirection;
	byte valLocFunction;
	byte valLocFuncPower;
	unsigned long valLocFuncBuf;
	unsigned char valSelectedMenuItemNo;
	unsigned char valConfigItemNo;
	word valCVAddr;
	byte valCVValue;
	byte mode_cv_seq;
	
	word valS88Data[8];
	
	//表示モード
	unsigned char mode_display;
	byte mode_display_temp;
	
	void clearTextBuf();
	void RefreshAll();
		
	public:
	
	DSLCDManager();
	void SetSensors(unsigned short inI_H, unsigned short inI_L, unsigned char inV_H, unsigned char inV_L, unsigned char inErrFlag);
	void SetSensorsErr(unsigned short inI, unsigned char inV);
	void Interval();
	void DisplayTop();
	void DisplayLocAddress();
	void DisplayAccAddress();
	unsigned char GetMode();
	
	void RegisterLocSpeed(word inLocAddr, word inSpeed);
	void RegisterLoc(word inLocAddr, unsigned char inDir, unsigned long inFuncBuf);
	void RegisterLocDirection(word inLocAddr, unsigned char inDir);
	void RegisterLocFunctionNormal(word inLocAddr, unsigned long inFuncBuf);
	void RegisterLocFunction(word inLocAddr, unsigned char inFuncNo, unsigned char inFuncPower);
	void RegisterPower(unsigned char inPower);
	void RegisterTurnout(word inAccAddr, unsigned char inDir);
	void RegisterMenu(byte inMenuItemNo);
	void RegisterConfig(byte inConfigItemNo);
	void RegisterTurnoutDir(word inAccAddr, unsigned char inDir);
	void RegisterMain();
	void RegisterAnalog();
	void RegisterProtocol(byte inProtocol);
	void RegisterInfo();
	word GetLocIDProtocol(byte address);
	void WriteLocAddr(word inAddr);
	void WriteAccAddr(word inAddr);
	void RegisterCVWrite(byte inSeq, word inAddr, byte inValue);
	void RegisterCVRead(byte inSeq, word inAddr, byte inValue);
	void ClearRow(byte inY);
	void StrCpyFromROM(char *outpText, const char *inpROMText, byte inLen);
	void ClearStringSpaces(char *inText, byte inLen);
	
	VoidFunction ClearDisplay;
	PrintFunction PrintString;
	PrintFunction PrintStringNOT;
	PrintFunction PrintStringBig;
	DrawFunction DrawIcon;
	DrawFunction DrawSpeedBar;
	GetFunction GetConfigData;
	

};

#endif
