/*********************************************************************
 * Sequence for Desktop Station main
 *
 * Copyright (C) 2015 Yaasan
 *
 */

#include <Arduino.h>
#include <EEPROM.h>
#include "DSCoreM_Type.h"
#include "DSCoreM_Common.h"
#include "DSCoreM.h"
#include "DSSequence.h"

#define PIN_SW_DN 6		//PD6
#define PIN_SW_UP 7		//PD7
#define PIN_SW_ENT 5	//PD5
#define PIN_SW_FNC 2	//PD2
#define PIN_SW_DIR 3	//PD3
#define PIN_SW_HOME 4	//PD4
#define PIN_SW_PWR	A3	//PC2
#define PIN_VR A2		//PC3



#define PRESS_THRESHOLD		10

byte gAccBuf[32];
DATA_LOC gLocBuf[4];

//Definition of functions
byte getAccBuf(word inNo);
void setAccBuf(word inNo, byte inData);
void setLocBuf(word inAddr, byte inDir, long inFuncBuf);
void getLocBuf(word inAddr, byte *ioDir, long *ioFuncBuf);

DSSequence::DSSequence()
{
	
	/* Peripherals */
	pinMode(PIN_SW_HOME, INPUT);
	pinMode(PIN_SW_DIR, INPUT);
	pinMode(PIN_SW_FNC, INPUT);
	pinMode(PIN_SW_ENT, INPUT);
	pinMode(PIN_SW_UP, INPUT);
	pinMode(PIN_SW_DN, INPUT);
	pinMode(PIN_SW_PWR, INPUT);
	digitalWrite(PIN_SW_PWR, HIGH);
	digitalWrite(PIN_SW_HOME, HIGH);
	digitalWrite(PIN_SW_DIR, HIGH);
	digitalWrite(PIN_SW_FNC, HIGH);
	digitalWrite(PIN_SW_ENT, HIGH);
	digitalWrite(PIN_SW_UP, HIGH);
	digitalWrite(PIN_SW_DN, HIGH);
	
	
	pinMode(PIN_VR, INPUT); // Input Speed ref
	digitalWrite(PIN_VR, LOW); // disable internal pullup
	analogReference(DEFAULT); // set analog reference 5V(VCC) 

	//Modes
	mode_protocol = 0;//DCC
	mode_seq = SQMODE_LOC;
	
	mode_depth = SQCMD_DEPTH_LOC;
	mode_depth_last = 0;
	
	//Function
	currentFnc = 0;
	currentFuncNo = 0;
	
	//Accessory buf clear
	for( byte j = 0; j < 32; j++)
	{
		gAccBuf[j] = 0;
	}
	
	//Loco buf clear
	for( byte k = 0; k < 4; k++)
	{
		gLocBuf[k].mAddress = 0;//Marklin Address Format
		gLocBuf[k].mDir = 0;
		gLocBuf[k].mFuncBuf = 0;
	}
	
	//Menu
	currentMenuNo = 0;
	
	//Configuration
	currentCfgNo = 0;
	
	//Previous values
	prev_dial = getDialStatus();
	prev_btns_SQ = getButtonStatus();
	
	//Control values
	NextLocoAddress = 3;
	NextLocoDirection = GO_FWD;
	NextAccAddress = 1;
	NextAccDirection = GO_STRAIGHT;
	NextPowerStatus = POWER_OFF;
	count_SelButton = 0;
	count_LocButton = 0;
	count_AdrButton = 0;
	
	//Load EEPROM
	//loadEEPROM();
	
	//NextLocoAddress = eepLocAddr;
	//NextAccAddress = eepAccAddr;
	
	//CV values
	cv_no = 1;
	cv_value = 0;
	
	//Update protocol data
	setProtocol();
	
}



void DSSequence::loadEEPROM()
{
	
	if( EEPROM.read(0) == 0xBB)
	{
		
		eepLocAddr = (EEPROM.read(3) << 8) + EEPROM.read(2);
		eepAccAddr = (EEPROM.read(5) << 8) + EEPROM.read(4);
		
	}	
	
}

void DSSequence::saveEEPROM(byte inMode)
{
	
	if( EEPROM.read(0) == 0xBB)
	{
		
		if( inMode == DSEEPROM_LOCADDR)
		{
			EEPROM.write(2, lowByte(NextLocoAddress));
			EEPROM.write(3, highByte(NextLocoAddress));
		}
		
		if( inMode ==  DSEEPROM_ACCADDR)
		{
			EEPROM.write(4, lowByte(NextAccAddress));
			EEPROM.write(5, highByte(NextAccAddress));
		}

		
		
		
	}
	else
	{
		//初期化
		EEPROM.write(0, 0xBB);
		
		//アドレス書き換え
		EEPROM.write(1, 0);
		EEPROM.write(2, lowByte(NextLocoAddress));
		EEPROM.write(3, highByte(NextLocoAddress));
		EEPROM.write(4, lowByte(NextAccAddress));
		EEPROM.write(5, highByte(NextAccAddress));
		
	}
	
}

byte DSSequence::getButtonStatus()
{
	
	byte aButton_val = 0;
	
	//Input Buttons
	bitWrite(aButton_val, 0, (digitalRead(PIN_SW_FNC) == 1) ? 0 : 1);//Dir
	bitWrite(aButton_val, 1, (digitalRead(PIN_SW_ENT) == 1) ? 0 : 1);//ENTER
	bitWrite(aButton_val, 2, (digitalRead(PIN_SW_HOME) == 1) ? 0 : 1);//HOME/ESC
	bitWrite(aButton_val, 3, (digitalRead(PIN_SW_UP) == 1) ? 0 : 1);//UP
	bitWrite(aButton_val, 4, (digitalRead(PIN_SW_DN) == 1) ? 0 : 1);//DOWN
	bitWrite(aButton_val, 5, (digitalRead(PIN_SW_DIR) == 1) ? 0 : 1);//Dir
	bitWrite(aButton_val, 6, (digitalRead(PIN_SW_PWR) == 1) ? 0 : 1);//Run/Stop
	
	return aButton_val;
	
}

word DSSequence::getDialStatus()
{
	word aData = analogRead(PIN_VR) & 0b1111111000;
	
	if( aData > 1023)
	{
		aData = 1023;
	}
	
	//Mask lower 3bits (1024 to 256)
	return aData;
}

byte DSSequence::GetModeDepth()
{
	return mode_depth;
}

bool DSSequence::Interval_Fast()
{
	bool aChange = false;
	word aDial_val;
	byte aButton_val = 0;
	
	//Input Analog (lower bit is ignored)
	gBuf_Dial = (getDialStatus() + gBuf_Dial) >> 1;
	aDial_val = gBuf_Dial;
	
	//Input Buttons
	aButton_val = getButtonStatus();
	
	//Check changes
	if( aDial_val != prev_dial)
	{
		//Set speed
		NextLocoSpeed = aDial_val;
		
		switch( mode_depth)
		{
		case SQCMD_DEPTH_ANA:
			if( NextLocoDirection == GO_FWD)
			{
				//REV
				analogWrite(PIN_PWMA, 255 - (NextLocoSpeed >> 2));
				analogWrite(PIN_PWMB, 255);
			}
			else
			{
				//FWD
				analogWrite(PIN_PWMB, 255 - (NextLocoSpeed >> 2));
				analogWrite(PIN_PWMA, 255);
			}
			
			if( ChangedTargetSpeed != NULL) {
				//表示のみ、パルス送信無し
				ChangedTargetSpeed(CurrentProtocol + NextLocoAddress, NextLocoSpeed, false);
			}
			break;
			
		default:
			if( ChangedTargetSpeed != NULL) {
				ChangedTargetSpeed(CurrentProtocol + NextLocoAddress, NextLocoSpeed, true);
			}
			break;
			
			
		}
		
		
		
		aChange = true;
		
	}
	else
	{
		//Nothing to do
	}
	
	//ボタンのチェック
	if( aButton_val != prev_btns)
	{
		//Serial.println(aButton_val,BIN);
		aChange = true;
		gIntervalChange = true;
		
		curr_btns_SQ = aButton_val;
		prev_btns_SQ = prev_btns;
		
	}
	else
	{
		//Nothing to do
	}	
	//Sequence main
	
	//Store previous values
	prev_dial = aDial_val;
	prev_btns = aButton_val;
	
	return aChange;
	
}	
	
void DSSequence::Interval_Slow()
{
	
	//if( gIntervalChange == true)
	//{
		
		switch( mode_depth)
		{
		case SQCMD_DEPTH_LOC:
			interval_Loc(curr_btns_SQ);
			
			break;
			
		case SQCMD_DEPTH_MENU:
			interval_Menu(curr_btns_SQ);
			
			break;
			
		case SQCMD_DEPTH_ACC:
			interval_Acc(curr_btns_SQ);
			
			break;
			
		case SQCMD_DEPTH_CVR:
			interval_ReadCV(curr_btns_SQ);
			
			break;
			
		case SQCMD_DEPTH_CVW:
			interval_WriteCV(curr_btns_SQ);
			break;
		
		case SQCMD_DEPTH_ANA:
			interval_Analog(curr_btns_SQ);
			break;
		
		}
		
		//Always run event
		interval_always(curr_btns_SQ);
		
		
		prev_btns_SQ = curr_btns_SQ;
		
		
	//	gIntervalChange = false;
	//}
	
}

/****************************************************
 
 Always run event
 
 ****************************************************/

void DSSequence::interval_always(byte inButtonStatus)
{
	
	//Check button changes
	if( inButtonStatus != prev_btns_SQ)
	{
		if( checkButton(prev_btns_SQ, inButtonStatus, SQBUTTON_RUN) == true )
		{
			if( (mode_depth != SQCMD_DEPTH_CVR) && (mode_depth != SQCMD_DEPTH_CVW) && (mode_depth != SQCMD_DEPTH_ANA))
			{
				clickRun();
			}
		}
		else if( checkButton(prev_btns_SQ, inButtonStatus, SQBUTTON_DIR) == true )
		{
			
			clickDir();
			
		}
	}	
	
}

void DSSequence::interval_Loc(byte inButtonStatus)
{
	
	//Check button changes
	if( inButtonStatus != prev_btns_SQ)
	{
		
		
		if( (gLocMode == 1) && (checkButton(prev_btns_SQ, inButtonStatus, SQBUTTON_UP) == true) )
		{
			//Clear counter
			count_SelButton = 255;
			
			currentFuncNo++;
			
			//F28超のとき
			if( currentFuncNo > 28)
			{
				currentFuncNo = 0;
			}
			
			
			/* 表示処理 */
			ChangedTargetFunc(CurrentProtocol + NextLocoAddress, currentFuncNo, bitRead(currentFnc, currentFuncNo), false, 1, 0);
			
		}
		else if( (gLocMode == 1) && (checkButton(prev_btns_SQ, inButtonStatus, SQBUTTON_DN) == true) )
		{
			//Clear counter
			count_SelButton = 255;
			
			//F0のとき
			if( currentFuncNo == 0 )
			{
				currentFuncNo = 28;
			}
			else
			{
				currentFuncNo--;
			}
			
			
			/* 表示処理 */
			ChangedTargetFunc(CurrentProtocol + NextLocoAddress, currentFuncNo, bitRead(currentFnc, currentFuncNo), false, 1, 0);
			
		}
		else if((gLocMode == 1) && ( checkButton(prev_btns_SQ, inButtonStatus, SQBUTTON_ENT) == true))
		{
			
			/*ファンクション操作*/
			if( bitRead(currentFnc, currentFuncNo) == 1)
			{
				bitWrite(currentFnc, currentFuncNo, 0);
			}
			else
			{
				bitWrite(currentFnc, currentFuncNo, 1);
			}
		
			//表示
			RegisterFunction(currentFuncNo);
			
			
			//EEPROM操作
			if( eepLocAddr != NextLocoAddress)
			{
				eepLocAddr = NextLocoAddress;
			//	saveEEPROM(DSEEPROM_LOCADDR);
			}
		}
		else if( (gLocMode == 0) &&  (checkButtonLeaved(prev_btns_SQ, inButtonStatus, SQBUTTON_UP) == true))
		{
			
			if( count_AdrButton <= PRESS_THRESHOLD)
			{
				//ファンクションバッファを退避
				setLocBuf(NextLocoAddress, NextLocoDirection ,currentFnc);
				
				//操作アドレス変更
				locaddress_temp = NextLocoAddress;
				plusLocAddress(&locaddress_temp, mode_protocol, 1);
				NextLocoAddress = locaddress_temp;
				
				//ファンクションバッファからあるものを取り出す
				getLocBuf(NextLocoAddress, &NextLocoDirection ,&currentFnc);
				
				//表示を更新
				ChangedTargetLocAddr(CurrentProtocol + NextLocoAddress, NextLocoDirection ,currentFnc);
				
				
				Serial.println(NextLocoAddress);
				
			}
			
			//Clear counter
			count_AdrButton = 0;
		}
		else if( (gLocMode == 0) &&  (checkButtonLeaved(prev_btns_SQ, inButtonStatus, SQBUTTON_DN) == true))
		{
			
			
			if( count_AdrButton <= PRESS_THRESHOLD)
			{
				//ファンクションバッファを退避
				setLocBuf(NextLocoAddress, NextLocoDirection ,currentFnc);
				
				//操作アドレス変更
				locaddress_temp = NextLocoAddress;
				minusLocAddress(&locaddress_temp, mode_protocol, 1);
				NextLocoAddress = locaddress_temp;
				
				//ファンクションバッファからあるものを取り出す
				getLocBuf(NextLocoAddress, &NextLocoDirection ,&currentFnc);
				
				//表示を更新
				ChangedTargetLocAddr(CurrentProtocol + NextLocoAddress, NextLocoDirection ,currentFnc);
			}
			
			//Clear counter
			count_AdrButton = 0;
		}
		else if( (gLocMode == 0) &&  (checkButtonLeaved(prev_btns_SQ, inButtonStatus, SQBUTTON_UP) == true) )
		{
			//Clear counter
			count_AdrButton = 0;
			
		}
		else if( (gLocMode == 0) &&  (checkButtonLeaved(prev_btns_SQ, inButtonStatus, SQBUTTON_DN) == true) )
		{
			//Clear counter
			count_AdrButton = 0;
			
		}
		else if(  (gLocMode == 0) && (checkButton(prev_btns_SQ, inButtonStatus, SQBUTTON_UP) == true) )
		{
			//Clear counter
			count_AdrButton = 1;
			
		}
		else if(  (gLocMode == 0) && (checkButton(prev_btns_SQ, inButtonStatus, SQBUTTON_DN) == true) )
		{
			//Clear counter
			count_AdrButton = 1;
			
		}
		else if( checkButton(prev_btns_SQ, inButtonStatus, SQBUTTON_FNC) == true )
		{
			
			//Clear counter
			if( gLocMode == 0)
			{
				gLocMode = 1;
			}
			else
			{
				gLocMode = 0;
			}
			
			/* 表示処理 */
			ChangedTargetFunc(CurrentProtocol + NextLocoAddress, currentFuncNo, bitRead(currentFnc, currentFuncNo), false, gLocMode, currentFnc);
			
		}
		else if( checkButton(prev_btns_SQ, inButtonStatus, SQBUTTON_ESC) == true )
		{
			//何もしない
			NextMode(SQCMD_DEPTH_MENU);
			
			//MENUに入る
			ChangedMenu(currentMenuNo);
		}
	}
	else
	{
		
		if( (gLocMode == 0) && (checkButtonContinuous(prev_btns_SQ, inButtonStatus, SQBUTTON_UP) == true ) && (count_AdrButton > 0))
		{
			//変化なしの時（連続押しの場合）
			count_AdrButton++;
			
			
			locaddress_temp = NextLocoAddress;
			
			if( count_AdrButton < PRESS_THRESHOLD)
			{
				//何もしない
				return;
			}
			else if( count_AdrButton < 50)
			{
				plusLocAddress(&locaddress_temp, mode_protocol, 1);
			}
			else if( count_AdrButton <150)
			{
				plusLocAddress(&locaddress_temp, mode_protocol, 10);
			}
			else if( count_AdrButton < 250)
			{
				plusLocAddress(&locaddress_temp, mode_protocol, 50);
			}
			else
			{
				count_AdrButton = 255;
				plusLocAddress(&locaddress_temp, mode_protocol, 100);
			}
			
			
			NextLocoAddress = locaddress_temp;
			
			//表示を更新
			ChangedTargetLocAddr(CurrentProtocol + NextLocoAddress, NextLocoDirection ,currentFnc);
			
		}
		else if( (gLocMode == 0) && (checkButtonContinuous(prev_btns_SQ, inButtonStatus, SQBUTTON_DN) == true ) && (count_AdrButton > 0))
		{
			count_AdrButton++;
			
			locaddress_temp = NextLocoAddress;
			
			if( count_AdrButton < PRESS_THRESHOLD)
			{
				//何もしない
				return;
			}
			else if( count_AdrButton < 50)
			{
				minusLocAddress(&locaddress_temp, mode_protocol, 1);
			}
			else if( count_AdrButton < 150)
			{
				minusLocAddress(&locaddress_temp, mode_protocol, 10);
			}
			else if( count_AdrButton < 250)
			{
				minusLocAddress(&locaddress_temp, mode_protocol, 50);
			}
			else
			{
				count_AdrButton = 255;
				minusLocAddress(&locaddress_temp, mode_protocol, 100);
			}
			
			NextLocoAddress = locaddress_temp;
			
			//表示を更新
			ChangedTargetLocAddr(CurrentProtocol + NextLocoAddress, NextLocoDirection ,currentFnc);
			
		}
		else if( checkButtonContinuous(prev_btns_SQ, inButtonStatus, SQBUTTON_ENT) == true )
		{
			if( count_SelButton < 255)
			{
				count_SelButton++;
				
			}
			
		}
		
	}
	
}

void DSSequence::interval_Analog(byte inButtonStatus)
{
	
	//Check button changes
	if( inButtonStatus != prev_btns_SQ)
	{
		
		
		if( checkButton(prev_btns_SQ, inButtonStatus, SQBUTTON_ESC) == true )
		{
			//何もしない
			NextMode(SQCMD_DEPTH_MENU);
			
			//MENUに入る
			ChangedMenu(currentMenuNo);
		}
	}
	else
	{
		
		
	}
	
}


void DSSequence::minusLocAddress(word *iopLocAddr, byte inProtocol, word inMinus)
{
	if( ((int)*iopLocAddr - (int)inMinus) <= 0)
	{
		if( inProtocol == 1)
		{
			//MM2
			*iopLocAddr = 255;
		}
		else
		{
			//DCC
			*iopLocAddr = 9999;
		}
	}
	else
	{
		*iopLocAddr = *iopLocAddr - inMinus;
	}
}

void DSSequence::plusLocAddress(word *iopLocAddr, byte inProtocol, word inPlus)
{
	
	if( inProtocol == 1)
	{
		if( (*iopLocAddr + inPlus) >= 255)
		{
			//MM2
			*iopLocAddr = 1;
		}
		else
		{
			*iopLocAddr = *iopLocAddr + inPlus;
		}
	}
	else
	{
		if( (*iopLocAddr + inPlus) > 9999)
		{
			//DCC
			*iopLocAddr = 1;
		}
		else
		{
			*iopLocAddr = *iopLocAddr + inPlus;
		}
	}
}

void DSSequence::minusAccAddress(word *iopAccAddr, byte inProtocol, word inMinus)
{
	if( ((int)*iopAccAddr - (int)inMinus) <= 0)
	{
		if( inProtocol == 1)
		{
			//MM2
			*iopAccAddr = 320;
		}
		else
		{
			//DCC
			*iopAccAddr = 2044;
		}
	}
	else
	{
		*iopAccAddr = *iopAccAddr - inMinus;
	}
}

void DSSequence::plusAccAddress(word *iopAccAddr, byte inProtocol, word inPlus)
{
	
	if( inProtocol == 1)
	{
		if( (*iopAccAddr + inPlus) >= 320)
		{
			//MM2
			*iopAccAddr = 1;
		}
		else
		{
			*iopAccAddr = *iopAccAddr + inPlus;
		}
	}
	else
	{
		if( (*iopAccAddr + inPlus) > 2044)
		{
			//DCC
			*iopAccAddr = 1;
		}
		else
		{
			*iopAccAddr = *iopAccAddr + inPlus;
		}
	}
}


void DSSequence::interval_Acc(byte inButtonStatus)
{
	
	//Check button changes
	if( inButtonStatus != prev_btns_SQ)
	{
		if( checkButton(prev_btns_SQ, inButtonStatus, SQBUTTON_UP) == true )
		{
			//Clear counter
			count_AdrButton = 0;
		}
		else if( checkButton(prev_btns_SQ, inButtonStatus, SQBUTTON_DN) == true )
		{
			//Clear counter
			count_AdrButton = 0;
			
		}
		else if( checkButtonLeaved(prev_btns_SQ, inButtonStatus, SQBUTTON_UP) == true)
		{
			
			if( count_AdrButton <= PRESS_THRESHOLD)
			{
				accaddress_temp = NextAccAddress;
				
				plusAccAddress(&accaddress_temp, mode_protocol, 1);
				NextAccAddress = accaddress_temp;
				NextAccDirection = getAccBuf(NextAccAddress);
				
				ChangedTurnout(CurrentACCProtocol + NextAccAddress, NextAccDirection, false);
				
			}
			//Clear counter
			count_AdrButton = 0;
		}
		else if( checkButtonLeaved(prev_btns_SQ, inButtonStatus, SQBUTTON_DN) == true)
		{
			
			if( count_AdrButton <= PRESS_THRESHOLD)
			{
				accaddress_temp = NextAccAddress;
				
				minusAccAddress(&accaddress_temp, mode_protocol, 1);
				NextAccAddress = accaddress_temp;
				NextAccDirection = getAccBuf(NextAccAddress);
				
				ChangedTurnout(CurrentACCProtocol + NextAccAddress, NextAccDirection, false);
			}
			//Clear counter
			count_AdrButton = 0;
		}
		else if( checkButton(prev_btns_SQ, inButtonStatus, SQBUTTON_ENT) == true )
		{
			
			//操作
			if( NextAccDirection == 0 )
			{
				NextAccDirection = 1;
			}
			else
			{
				NextAccDirection = 0;
			}
			
			ChangedTurnout(CurrentACCProtocol + NextAccAddress, NextAccDirection, true);
			
			//内部バッファに記録する
			setAccBuf(NextAccAddress, NextAccDirection);
			
			//EEPROM操作
			if( eepAccAddr != NextAccAddress)
			{
				//eepAccAddr = NextAccAddress;
				//saveEEPROM(DSEEPROM_ACCADDR);
			}
			
		}
		else if( checkButton(prev_btns_SQ, inButtonStatus, SQBUTTON_ESC) == true )
		{
			NextMode(SQCMD_DEPTH_MENU);
			
			//MENUに入る
			ChangedMenu(currentMenuNo);
		}
	}
	else
	{
		//変化なしの時（連続押しの場合）
		
		if( checkButtonContinuous(prev_btns_SQ, inButtonStatus, SQBUTTON_UP) == true )
		{
			count_AdrButton++;
			
			accaddress_temp = NextAccAddress;
			
			if( count_AdrButton < PRESS_THRESHOLD)
			{
				//何もしない
				return;
			}
			else if( count_AdrButton < 50)
			{
				plusAccAddress(&accaddress_temp, mode_protocol, 1);
			}
			else if( count_AdrButton <150)
			{
				plusAccAddress(&accaddress_temp, mode_protocol, 10);
			}
			else if( count_AdrButton < 250)
			{
				plusAccAddress(&accaddress_temp, mode_protocol, 20);
			}
			else
			{
				count_AdrButton = 255;
				plusAccAddress(&accaddress_temp, mode_protocol, 50);
			}
			
			
			NextAccAddress = accaddress_temp;
			
			ChangedTurnout(CurrentACCProtocol + NextAccAddress, NextAccDirection, false);
			
		}
		else if( checkButtonContinuous(prev_btns_SQ, inButtonStatus, SQBUTTON_DN) == true )
		{
			count_AdrButton++;
			
			accaddress_temp = NextAccAddress;
			
			if( count_AdrButton < PRESS_THRESHOLD)
			{
				//何もしない
				return;
			}
			else if( count_AdrButton < 50)
			{
				minusAccAddress(&accaddress_temp, mode_protocol, 1);
			}
			else if( count_AdrButton < 150)
			{
				minusAccAddress(&accaddress_temp, mode_protocol, 10);
			}
			else if( count_AdrButton < 250)
			{
				minusAccAddress(&accaddress_temp, mode_protocol, 20);
			}
			else
			{
				count_AdrButton = 255;
				minusAccAddress(&accaddress_temp, mode_protocol, 50);
			}
			
			NextAccAddress = accaddress_temp;
			
			ChangedTurnout(CurrentACCProtocol + NextAccAddress, NextAccDirection, false);
			
		}
		else if( checkButtonContinuous(prev_btns_SQ, inButtonStatus, SQBUTTON_ENT) == true )
		{
			if( count_SelButton < 255)
			{
				count_SelButton++;
				
			}
			
		}
		
	}
	
	
}

void DSSequence::interval_Menu(byte inButtonStatus)
{
	
	//Check button changes
	if( inButtonStatus != prev_btns_SQ)
	{
		if( checkButton(prev_btns_SQ, inButtonStatus, SQBUTTON_DN) == true )
		{
			//Clear counter
			count_SelButton = 255;
			
			if( currentMenuNo >= 5)
			{
				currentMenuNo = 0;
			}
			else
			{
				currentMenuNo = currentMenuNo + 1;
			}
			
			ChangedMenu(currentMenuNo);
		}
		else if( checkButton(prev_btns_SQ, inButtonStatus, SQBUTTON_UP) == true )
		{
			//Clear counter
			count_SelButton = 255;
			
			
			if( currentMenuNo <= 0)
			{
				currentMenuNo = 5;
			}
			else
			{
				currentMenuNo = currentMenuNo - 1;
			}
			
			ChangedMenu(currentMenuNo);
			
		}
		else if( checkButton(prev_btns_SQ, inButtonStatus, SQBUTTON_ENT) == true )
		{
			//切り替え
			switch(currentMenuNo)
			{
			
			case 1:
				//Accessory
				NextMode(SQCMD_DEPTH_ACC);
				
				//Refresh
				DisplayClear();
				ChangedTurnout(CurrentACCProtocol + NextAccAddress, NextAccDirection, false);
				
				break;
			case 2:
				//CV Read
				NextMode(SQCMD_DEPTH_CVR);
				
				//Refresh
				DisplayClear();
				
				//Read CV Func
				currentCVSeq = 0;
				ReadCVFunc(currentCVSeq, cv_no, cv_value);
				break;
			case 3:
				//CV Write
				NextMode(SQCMD_DEPTH_CVW);
				
				//Refresh
				DisplayClear();
				
				//Read CV Func
				currentCVSeq = 0;
				WriteCVFunc(currentCVSeq, cv_no, cv_value);
				
				break;
			case 4:
				//Analog Mode
				NextMode(SQCMD_DEPTH_ANA);
				
				//Refresh
				DisplayClear();
				ChangedAnalog();
				break;
			case 5:
				//Protocol change
				
				if( mode_protocol == 0)
				{
					//MM2 mode
					mode_protocol = 1;
				}
				else
				{
					//DCC mode
					mode_protocol = 0;
				}
				
				setProtocol();
				ChangedProtocol(mode_protocol);
				
				//何もしない
				NextMode(SQCMD_DEPTH_MENU);
				
				
				
				break;
			case 6:
				break;
			case 0:
			default:
				//Locomotive
				NextMode(SQCMD_DEPTH_LOC);
				
				//Refresh
				DisplayClear();
				ChangedMain();
				break;
				
			}
			
		}
		else if( checkButton(prev_btns_SQ, inButtonStatus, SQBUTTON_ESC) == true )
		{
			/* 何もしない */
			
		}

	}
	
}


void DSSequence::interval_Config(byte inButtonStatus)
{
	
}



void DSSequence::interval_ReadCV(byte inButtonStatus)
{
	
	//Check button changes
	if( inButtonStatus == prev_btns_SQ)
	{
		if( (checkButtonContinuous(prev_btns_SQ, inButtonStatus, SQBUTTON_UP) == true )  && (count_AdrButton > 0))
		{
			//Clear counter
			count_AdrButton++;
			count_SelButton = 255;
			
			if( count_AdrButton < PRESS_THRESHOLD)
			{
				//何もしない
				return;
			}
			
			if( currentCVSeq == 1)
			{
				return;
			}
			
			if( cv_no >= 1024)
			{
				cv_no = 1;
			}
			else
			{
				cv_no = cv_no + 1;
			}
			
			ReadCVFunc(currentCVSeq, cv_no, cv_value);
		}
		else if( (checkButtonContinuous(prev_btns_SQ, inButtonStatus, SQBUTTON_DN) == true ) && (count_AdrButton > 0))
		{
			//Clear counter
			count_AdrButton++;
			
			if( count_AdrButton < PRESS_THRESHOLD)
			{
				//何もしない
				return;
			}
			
			if( currentCVSeq == 1)
			{
				return;
			}
			
			
			if( cv_no <= 1)
			{
				cv_no = 1024;
			}
			else
			{
				cv_no = cv_no - 1;
			}
			
			ReadCVFunc(currentCVSeq, cv_no, cv_value);
			
		}
	}
	else
	{
		//ボタンの状態が変化したとき
		
		if( checkButton(prev_btns_SQ, inButtonStatus, SQBUTTON_ENT) == true )
		{
			//切り替え
			switch(currentCVSeq)
			{
			
			case 0:
				//Address to Read(CVR)
				currentCVSeq++;
				
				//Read CV Func
				ReadCVFunc(currentCVSeq, cv_no, cv_value);
				break;
			case 1:
				//Go to Menu
				mode_depth = SQCMD_DEPTH_MENU;
				
				//Refresh
				DisplayClear();
				ChangedMenu(currentMenuNo);
				
				break;
			}
			
		}
		else if( checkButton(prev_btns_SQ, inButtonStatus, SQBUTTON_ESC) == true )
		{
			
			if( currentCVSeq == 0)
			{
				DisplayClear();
				
				
				ChangedMenu(currentMenuNo);
				mode_depth = SQCMD_DEPTH_MENU;
			}
			else
			{
				currentCVSeq--;
				//Read CV Func
				ReadCVFunc(currentCVSeq, cv_no, cv_value);
			}
		}
		else if( checkButtonLeaved(prev_btns_SQ, inButtonStatus, SQBUTTON_UP) == true )
		{
			if( count_AdrButton <= PRESS_THRESHOLD)
			{
				if( currentCVSeq == 1)
				{
					return;
				}
				
				if( cv_no >= 1024)
				{
					cv_no = 1;
				}
				else
				{
					cv_no = cv_no + 1;
				}
				
				ReadCVFunc(currentCVSeq, cv_no, cv_value);
			}
			
			
			//Clear counter
			count_AdrButton = 0;
			
		}
		else if( checkButtonLeaved(prev_btns_SQ, inButtonStatus, SQBUTTON_DN) == true )
		{
			if( count_AdrButton <= PRESS_THRESHOLD)
			{
				
				if( currentCVSeq == 1)
				{
					return;
				}
				
				
				if( cv_no <= 1)
				{
					cv_no = 1024;
				}
				else
				{
					cv_no = cv_no - 1;
				}
				
				ReadCVFunc(currentCVSeq, cv_no, cv_value);
				
			}
			
			//Clear counter
			count_AdrButton = 0;
			
		}
		else if( checkButton(prev_btns_SQ, inButtonStatus, SQBUTTON_UP) == true )
		{
			//Clear counter
			count_AdrButton = 1;
			
		}
		else if( checkButton(prev_btns_SQ, inButtonStatus, SQBUTTON_DN) == true )
		{
			//Clear counter
			count_AdrButton = 1;
			
		}

	}
	
}



void DSSequence::interval_WriteCV(byte inButtonStatus)
{
	
	//Check button changes
	if( inButtonStatus == prev_btns_SQ)
	{
		
		if( (checkButtonContinuous(prev_btns_SQ, inButtonStatus, SQBUTTON_UP) == true )  && (count_AdrButton > 0))
		{
			//Clear counter
			count_AdrButton++;
			
			if( count_AdrButton < PRESS_THRESHOLD)
			{
				//何もしない
				return;
			}
			
			if( currentCVSeq == 0)
			{
				if( cv_no >= 1024)
				{
					cv_no = 1;
				}
				else
				{
					cv_no = cv_no + 1;
				}
			}
			else if (currentCVSeq == 1)
			{
				if( cv_value >= 255)
				{
					cv_value = 0;
				}
				else
				{
					cv_value = cv_value + 1;
				}
			}
			
			WriteCVFunc(currentCVSeq, cv_no, cv_value);
		}
		else if( (checkButtonContinuous(prev_btns_SQ, inButtonStatus, SQBUTTON_DN) == true ) && (count_AdrButton > 0))
		{
			//Clear counter
			count_AdrButton++;
			
			if( count_AdrButton < PRESS_THRESHOLD)
			{
				//何もしない
				return;
			}
			
			if( currentCVSeq == 0)
			{
				if( cv_no <= 1)
				{
					cv_no = 1024;
				}
				else
				{
					cv_no = cv_no - 1;
				}
			}
			else if (currentCVSeq == 1)
			{
				
				if( cv_value <= 0)
				{
					cv_value = 255;
				}
				else
				{
					cv_value = cv_value - 1;
				}
			}
			
			
			WriteCVFunc(currentCVSeq, cv_no, cv_value);
			
		}
	}
	else
	{
		if( checkButton(prev_btns_SQ, inButtonStatus, SQBUTTON_ENT) == true )
		{
			//切り替え
			switch(currentCVSeq)
			{
			
			case 0:
				//Write CV
				currentCVSeq++;
				
				//Write CV Func
				WriteCVFunc(currentCVSeq, cv_no, cv_value);
				break;
			case 1:
				//Write CV
				currentCVSeq++;
				
				//Write CV Func
				WriteCVFunc(currentCVSeq, cv_no, cv_value);
				break;
			case 2:
				//Go to Menu
				mode_depth = SQCMD_DEPTH_MENU;
				
				//Refresh
				DisplayClear();
				ChangedMenu(currentMenuNo);
				
				break;
			}
			
		}
		else if( checkButton(prev_btns_SQ, inButtonStatus, SQBUTTON_ESC) == true )
		{
			
			if( currentCVSeq == 0)
			{
				DisplayClear();
				
				
				ChangedMenu(currentMenuNo);
				mode_depth = SQCMD_DEPTH_MENU;
			}
			else
			{
				currentCVSeq--;
				//Read CV Func
				WriteCVFunc(currentCVSeq, cv_no, cv_value);
			}
		}
		else if( checkButtonLeaved(prev_btns_SQ, inButtonStatus, SQBUTTON_UP) == true )
		{
			if( count_AdrButton <= PRESS_THRESHOLD)
			{
				if( currentCVSeq == 0)
				{
					if( cv_no >= 1024)
					{
						cv_no = 1;
					}
					else
					{
						cv_no = cv_no + 1;
					}
				}
				else if (currentCVSeq == 1)
				{
					if( cv_value >= 255)
					{
						cv_value = 0;
					}
					else
					{
						cv_value = cv_value + 1;
					}
				}
				
				WriteCVFunc(currentCVSeq, cv_no, cv_value);
			}
			
			
			//Clear counter
			count_AdrButton = 0;
			
		}
		else if( checkButtonLeaved(prev_btns_SQ, inButtonStatus, SQBUTTON_DN) == true )
		{
			if( count_AdrButton <= PRESS_THRESHOLD)
			{
				if( currentCVSeq == 0)
				{
					if( cv_no <= 1)
					{
						cv_no = 1024;
					}
					else
					{
						cv_no = cv_no - 1;
					}
				}
				else if (currentCVSeq == 1)
				{
					
					if( cv_value <= 0)
					{
						cv_value = 255;
					}
					else
					{
						cv_value = cv_value - 1;
					}
				}
				
				WriteCVFunc(currentCVSeq, cv_no, cv_value);
			}
			
			//Clear counter
			count_AdrButton = 0;
			
		}
		else if( checkButton(prev_btns_SQ, inButtonStatus, SQBUTTON_UP) == true )
		{
			//Clear counter
			count_AdrButton = 1;
			
		}
		else if( checkButton(prev_btns_SQ, inButtonStatus, SQBUTTON_DN) == true )
		{
			//Clear counter
			count_AdrButton = 1;
			
		}
	}
	
}


void DSSequence::interval_Info(byte inButtonStatus)
{
	
	//Check button changes
	if( inButtonStatus != prev_btns_SQ)
	{
		if( checkButton(prev_btns_SQ, inButtonStatus, SQBUTTON_UP) == true )
		{
			//何もしない
			
		}
		else if( checkButton(prev_btns_SQ, inButtonStatus, SQBUTTON_DN) == true )
		{
			//何もしない
			
		}
		else if( checkButton(prev_btns_SQ, inButtonStatus, SQBUTTON_ENT) == true )
		{
			//何もしない
		}
		else if( checkButton(prev_btns_SQ, inButtonStatus, SQBUTTON_ESC) == true )
		{
			DisplayClear();
			
			
			ChangedMenu(currentMenuNo);
			mode_depth = SQCMD_DEPTH_MENU;
		}

	}
	
}

void DSSequence::setProtocol()
{
	switch(mode_protocol)
	{
	case 1:
		//MM2
		CurrentProtocol = 0x0000;
		CurrentACCProtocol = 0x2FFF;
		
		//リミット処理
		if( NextLocoAddress > 255)
		{
			NextLocoAddress = 255;
		}
		if( NextAccAddress > 320)
		{
			NextAccAddress = 320;
		}
		
		break;
	case 0:
		CurrentProtocol = 0xC000;
		CurrentACCProtocol = 0x37FF;
		break;
		
	}
	
}

bool DSSequence::checkButton(byte inBuf, byte inCurrent, byte inButtonBit)
{

	if( ((inBuf & inButtonBit) == 0) && ((inCurrent & inButtonBit) > 0))
	{
		return true;
	}
	else
	{
		return false;
	}
	
}

bool DSSequence::checkButtonLeaved(byte inBuf, byte inCurrent, byte inButtonBit)
{

	if( ((inBuf & inButtonBit) > 0) && ((inCurrent & inButtonBit) == 0))
	{
		return true;
	}
	else
	{
		return false;
	}
	
}

bool DSSequence::checkButtonContinuous(byte inBuf, byte inCurrent, byte inButtonBit)
{

	if( ((inBuf & inButtonBit) > 0) && ((inCurrent & inButtonBit) > 0))
	{
		return true;
	}
	else
	{
		return false;
	}
	
}

void DSSequence::RegisterDir()
{
	//Call click dir
	clickDir();
}

void DSSequence::RegisterSpeed(word inSpeed)
{
	
	ChangedTargetSpeed(CurrentProtocol + NextLocoAddress, inSpeed, true);
	
}

void DSSequence::RegisterFunction(byte inFuncNo)
{
	ChangedTargetFunc(CurrentProtocol + NextLocoAddress, inFuncNo, bitRead(currentFnc, inFuncNo), true, 1, 0);
}


void DSSequence::NextMode(byte inMode)
{
	mode_depth_last = mode_depth;
	mode_depth = inMode;
}

void DSSequence::clickDir()
{
	
	NextLocoDirection = (NextLocoDirection == GO_FWD) ? GO_REV : GO_FWD;
	
	ChangedTargetDir(CurrentProtocol + NextLocoAddress, NextLocoDirection, true);
	
	
}

void DSSequence::clickRun()
{
	
	switch(NextPowerStatus)
	{
	case POWER_OFF:
		
		//エラーチェック
		NextPowerStatus = POWER_ON;
		break;
	case POWER_ON:
		NextPowerStatus = POWER_OFF;
		break;
	}
	
	ChangedPowerStatus(NextPowerStatus);
	
	//Update protocol data
	setProtocol();
	
	//Wait for LCD message
	//delay(TIME_RETURN >> 1);
	

}

byte getAccBuf(word inNo)
{
	byte aRes = 0;
	
	if( inNo < 256)
	{
		aRes = bitRead(gAccBuf[inNo >> 3], inNo & 0b111);
	}
	else
	{
		aRes = 0;
	}
	
	return aRes;
}

void setAccBuf(word inNo, byte inData)
{
	
	if( inNo < 256)
	{
		bitWrite(gAccBuf[inNo >> 3], inNo & 0b111, inData);
	}
}


void getLocBuf(word inAddr, byte *ioDir, long *ioFuncBuf)
{
	byte aRes = 0;
	
	for( byte k = 0; k < 4; k++)
	{
		if( gLocBuf[k].mAddress == inAddr)
		{
			*ioDir = gLocBuf[k].mDir;
			*ioFuncBuf = gLocBuf[k].mFuncBuf;
			aRes = 1;
			break;
		}
	}
	
	//If not found, set zero.
	if( aRes == 0)
	{
		*ioDir = 0;
		*ioFuncBuf = 0;
	}
}

void setLocBuf(word inAddr, byte inDir, long inFuncBuf)
{
	for( byte k = 0; k < 4; k++)
	{
		if( (gLocBuf[k].mAddress == 0) || (gLocBuf[k].mAddress == inAddr))
		{
			if( (inFuncBuf != 0) || (inDir == 1))
			{
				gLocBuf[k].mAddress = inAddr;
				gLocBuf[k].mDir = inDir;
				gLocBuf[k].mFuncBuf = inFuncBuf;
				Serial.println(inAddr);
				break;
			}
		}
	}
}

